"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var forms_1 = require("@angular/forms");
var Ng2SsComponent = /** @class */ (function () {
    function Ng2SsComponent() {
        var _this = this;
        this.openSSOptions = false;
        this.currTabIndex = 0;
        this.currOptionIndex = 0;
        this.onOpen = new core_1.EventEmitter();
        this.onClose = new core_1.EventEmitter();
        this.onChange = new core_1.EventEmitter();
        /**
         * Function to search for the target class and close the dropdown
         * if the click was outside the component
         */
        this.onOutClick = function (event) {
            if (event.keyCode === 27) {
                _this.closeSerSel();
            }
            else {
                var elFound = false;
                for (var i = 0; i < event.path.length; i++) {
                    var classList = event.path[i].classList;
                    if (classList && classList.length > 0 && classList.value.indexOf('ng2-ss') > -1) {
                        elFound = true;
                        break;
                    }
                }
                if (!elFound) {
                    _this.closeSerSel();
                }
            }
        };
        /**
         * Custom Form interface methods
         */
        this.propagateChange = function (_) { };
    }
    Ng2SsComponent_1 = Ng2SsComponent;
    /**
     * On init, make sure inputs are usable, pre-select item based on inputs,
     * remember the option or tab selection
     */
    Ng2SsComponent.prototype.ngOnInit = function () {
        this.defaultIndex = this.defaultIndex || 0;
        this.defaultTab = this.defaultTab || 0;
        if (this.data && this.data.length > 1) {
            this.ssDisplayValue = this.data[this.defaultTab].data[this.defaultIndex];
        }
        else if (this.data && this.data.length === 1) {
            this.ssDisplayValue = this.data[0].data[this.defaultIndex];
            this.defaultTab = 0;
        }
        this.currTabIndex = this.defaultTab;
        this.currOptionIndex = this.defaultIndex;
    };
    /**
     * On change, if default index was changed, select the element.
     * If default tab was changes, switch the tab
     * @param changes
     */
    Ng2SsComponent.prototype.ngOnChanges = function (changes) {
        if (changes.defaultIndex && !changes.defaultIndex.firstChange) {
            this.ngOnInit();
        }
        if (changes.defaultTab && !changes.defaultTab.firstChange) {
            this.showTab(this.defaultTab);
            this.ngOnInit();
        }
    };
    /**
     * Function to open the options dropdown, emit an on-open event
     * and focus on the search field in the dropdown
     */
    Ng2SsComponent.prototype.openOptions = function () {
        this.openSSOptions = !this.openSSOptions;
        this.addListeners();
        if (this.onOpen.observers.length > 0) {
            this.onOpen.emit();
        }
        if (this.openSSOptions) {
            setTimeout(this.focusSearch, 800);
        }
    };
    /**
     * Function to close the dropdown, remove event listeners
     * and emit a on-close event
     */
    Ng2SsComponent.prototype.closeSerSel = function () {
        this.openSSOptions = false;
        this.removeListeners();
        if (this.onClose.observers.length > 0) {
            this.onClose.emit();
        }
    };
    /**
     * Funciton to select an item from the dropdown.
     * Once selected, remember the selection, emit an on-change event
     * and close the dropdown. If mutli tabs present, remember the tab.
     * @param item
     * @param idx
     * @param tab
     */
    Ng2SsComponent.prototype.selectSSItem = function (item, idx, tab) {
        this.currOptionIndex = idx;
        this.defaultIndex = idx;
        this.defaultTab = tab;
        if (typeof (item) === 'string' && item === 'default') {
        }
        else {
            this.ssDisplayValue = item;
        }
        if (this.onChange.observers.length > 0) {
            this.onChange.emit(item);
        }
        this.ssSearchKey = '';
        this.propagateChange(item);
        this.closeSerSel();
    };
    /**
     * Function to switch tabs and keep the focus on the search text field
     * @param i
     */
    Ng2SsComponent.prototype.showTab = function (i) {
        if (this.currTabIndex !== i) {
            this.currTabIndex = i;
            this.currOptionIndex = null;
            this.focusSearch();
        }
    };
    /**
     * Function to focus on the search text field
     */
    Ng2SsComponent.prototype.focusSearch = function () {
        if (this.searchInput) {
            this.searchInput.nativeElement.focus();
        }
    };
    /**
     * Function to remove event listeners
     */
    Ng2SsComponent.prototype.removeListeners = function () {
        document.removeEventListener('click', this.onOutClick);
        document.removeEventListener('keyup', this.onOutClick);
    };
    /**
     * Function to add listeners to close dropdown on outside click
     */
    Ng2SsComponent.prototype.addListeners = function () {
        document.addEventListener('click', this.onOutClick);
        document.addEventListener('keyup', this.onOutClick);
    };
    /**
     * Custom Form interface methods
     * @param value
     */
    Ng2SsComponent.prototype.writeValue = function (value) {
        if (!!value) {
            this.ssDisplayValue = value;
        }
    };
    /**
     * Custom Form interface methods
     * @param fn
     */
    Ng2SsComponent.prototype.registerOnChange = function (fn) {
        this.propagateChange = fn;
    };
    /**
     * Custom Form interface methods
     */
    Ng2SsComponent.prototype.registerOnTouched = function () {
    };
    __decorate([
        core_1.ViewChild('optionSearch'),
        __metadata("design:type", core_1.ElementRef)
    ], Ng2SsComponent.prototype, "searchInput", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], Ng2SsComponent.prototype, "width", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], Ng2SsComponent.prototype, "data", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], Ng2SsComponent.prototype, "defaultIndex", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], Ng2SsComponent.prototype, "defaultTab", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], Ng2SsComponent.prototype, "readOnly", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], Ng2SsComponent.prototype, "onOpen", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], Ng2SsComponent.prototype, "onClose", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", core_1.EventEmitter)
    ], Ng2SsComponent.prototype, "onChange", void 0);
    Ng2SsComponent = Ng2SsComponent_1 = __decorate([
        core_1.Component({
            selector: 'search-select',
            // templateUrl: './ng2-ss.component.html',
            template: "\n  <div class=\"ng2-ss\" [ngStyle]=\"{'width': width || '350px'}\">\n    <div name=\"ng2-ss-disp\" id=\"ng2-ss-disp\" class=\"ng2-ss__main-input\" \n    (click)=\"openOptions()\" [ngClass]=\"{'ng2-ss__main-input__active': openSSOptions}\">\n      <span>{{ssDisplayValue?.label || 'Choose an option'}}</span>\n    </div>\n    <div class=\"ng2-ss__main-input__arrow\" (click)=\"openOptions()\">\n      <span *ngIf=\"!openSSOptions\">&#9660;</span>\n      <span *ngIf=\"openSSOptions\">&#9650;</span>\n    </div>\n    <div *ngIf=\"openSSOptions\" class=\"ng2-ss__options\">\n      <div class=\"ng2-ss__options__header\">\n          Select an option\n          <span (click)=\"closeSerSel()\">&#10006;</span>\n      </div>\n      <div class=\"ng2-ss__options__search\">\n          <input type=\"text\" name=\"ng2-ss-search\" id=\"ng2-ss-search\" #optionSearch\n          class=\"ng2-ss__options__search__input\" [(ngModel)]=\"ssSearchKey\" />\n          <div *ngIf=\"data.length > 1\" class=\"ng2-ss__options__search__tabs__holder\">\n            <div *ngFor=\"let item of data; let idx = index;\" (click)=\"showTab(idx)\" class=\"ng2-ss__options__search__tabs\" [ngClass]=\"{'ng2-ss__options__search__tabs__active': currTabIndex === idx}\">\n              <span title=\"{{item.title}}\">{{item.title}}</span>\n            </div>\n          </div>\n      </div>\n      <div class=\"ng2-ss__options__holder\">\n        <div *ngIf=\"!data || data?.length < 1\" (click)=\"selectSSItem(item)\" class=\"ng2-ss__options__holder__item\">\n          <span>&nbsp;&nbsp;&nbsp;</span>Loading ...\n        </div>\n        <div *ngFor=\"let item of data; let i = index;\">\n          <div *ngIf=\"currTabIndex === i\">\n            <div *ngFor=\"let d of item.data | ng2Ss : ssSearchKey; let j = index;\" (click)=\"selectSSItem(d, j, i)\"\n            class=\"ng2-ss__options__holder__item\" [ngClass]=\"{'ng2-ss__options__holder__item__active': ssDisplayValue.label === d.label}\">\n            <div *ngIf=\"d === -1\" style=\"text-align:center;\">No matches found</div>\n            <span title=\"{{d.label}}\"  *ngIf=\"d !== -1\">\n              <span *ngIf=\"ssDisplayValue.label === d.label\">&#10004;</span>\n              <span *ngIf=\"ssDisplayValue.label !== d.label\">&nbsp;&nbsp;&nbsp;</span>\n              {{d.label}}\n            </span>\n          </div>\n        </div>\n        </div>\n      </div>\n    </div>\n  </div>\n  ",
            // styleUrls: ['./ng2-ss.component.less'],
            providers: [
                {
                    provide: forms_1.NG_VALUE_ACCESSOR,
                    useExisting: core_1.forwardRef(function () { return Ng2SsComponent_1; }),
                    multi: true
                }
            ]
        }),
        __metadata("design:paramtypes", [])
    ], Ng2SsComponent);
    return Ng2SsComponent;
    var Ng2SsComponent_1;
}());
exports.Ng2SsComponent = Ng2SsComponent;
//# sourceMappingURL=ng2-ss.component.js.map